/* Pin Configuration
 *
 * PB0: IRS Left
 * PB1: BD-IRQ
 * PB2: BD-CS / PCINT2
 * PB3: MOSI
 * PB4: MISO
 * PB5: SCK
 * PB6: XTAL1
 *
 * PC0:	IRR Right
 * PC1: IRR Barrier
 * PC2: IRR Left
 * PC3: IRR Center
 * PC6: Reset
 *
 * PD5: IRS Barrier
 * PD6: IRS Center
 * PD7: IRS Right
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <stdint.h>

#include "ir_data.h"
#include "spi.h"

#define PIN_IRS_L	PB0
#define PIN_IRS_C	PD6
#define PIN_IRS_R	PD7
#define PIN_IRS_B	PD5

#define PIN_IRR_L	PC2
#define PIN_IRR_C	PC3
#define PIN_IRR_R	PC0
#define PIN_IRR_B	PC1

#define PIN_IRQ		PB1
#define PIN_CS		PB2
#define PIN_MISO	PB4

IRData data;

int main()
{
	// set data directions, 0 = input, 1 = output
	DDRB = (1 << PIN_IRQ) | (1 << PIN_IRS_R);
	DDRC = 0;
	DDRD = (1 << PIN_IRS_C) | (1 << PIN_IRS_L) | (1 << PIN_IRS_B);

	// init data
	data.level[0] = 0x10;
	data.level[1] = 0x20;
	data.level[2] = 0x40;
	data.level[3] = 0x80;

	SPIInit();

	// init ADC
	ADCSRA = (1 << ADEN) | (1 << ADIF) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);	// Enable, prescaler 128 = ~93kHz
	DIDR0 = 0x0F;

	PORTD |= (1 << PIN_IRS_B);

	uint8_t i;
	uint16_t ch[4];

	sei();

	while(1)
	{
		SPIExchange(data.exchange.miso, data.exchange.mosi, IR_DATA_SIZE);

		for(i = 0; i < 4; i++)
			ch[i] = 0;

		uint8_t iterations;
		for(iterations = 0; iterations < 10; iterations++)
		{
			for(i = 0; i < 4; i++)
			{
				ADMUX = i;
				ADCSRA |= (1 << ADSC);
				while(ADCSRA & (1 << ADSC));
				ch[i] += ADC;
			}
		}

		for(i = 0; i < 4; i++)
		{
			data.level[i] = ch[i]/iterations;
			data.level[i] *= 20;
		}

		if(data.level[IR_BARRIER] < 12000)
			PORTB &= ~(1 << PIN_IRQ);
		else
			PORTB |= (1 << PIN_IRQ);
	}

	return 0;
}
