/*
 * commands.h
 *
 *  Created on: 24.05.2010
 *      Author: AndreR
 */

#ifndef __COMMANDS_H__
#define __COMMANDS_H__

#include "stm32f10x.h"

#define PACKED __attribute__ ((packed))

// SECTIONS
#define SECTION_SYSTEM 0
#define SECTION_DECT 1
#define SECTION_MOTOR 2
#define SECTION_KICKER 4
#define SECTION_MOVEMENT 5
#define SECTION_MULTICAST 6

// SYSTEM
#define CMD_SYSTEM_STATUS_MOVEMENT 3
#define LEN_SYSTEM_STATUS_MOVEMENT 12

#define CMD_SYSTEM_POWER_LOG 2
#define LEN_SYSTEM_POWER_LOG 14

#define CMD_SYSTEM_SET_IDENTITY 4
#define LEN_SYSTEM_SET_IDENTITY 27

#define CMD_SYSTEM_ANNOUNCEMENT 5
#define LEN_SYSTEM_ANNOUNCEMENT 12

#define CMD_SYSTEM_PING 6
#define LEN_SYSTEM_PING 4

#define CMD_SYSTEM_PONG 7
#define LEN_SYSTEM_PONG 4

#define CMD_SYSTEM_BIG_PING 8
#define LEN_SYSTEM_BIG_PING 200

#define CMD_SYSTEM_SET_LOGS 9
#define LEN_SYSTEM_SET_LOGS 9

// KICKER
#define CMD_KICKER_KICKV1 1
#define LEN_KICKER_KICKV1 4

#define CMD_KICKER_STATUS 2
#define LEN_KICKER_STATUS 12

#define CMD_KICKER_SET_AUTOLOAD 3
#define LEN_KICKER_SET_AUTOLOAD 2

#define CMD_KICKER_SET_MAX_CAP_LEVEL 4
#define LEN_KICKER_SET_MAX_CAP_LEVEL 2

#define CMD_KICKER_CHARGE 5
#define LEN_KICKER_CHARGE 2

#define CMD_KICKER_SET_DUTY_CYCLE 6
#define LEN_KICKER_SET_DUTY_CYCLE 4

#define CMD_KICKER_KICKV2 7
#define LEN_KICKER_KICKV2 4

#define CMD_KICKER_STATUSV2 8
#define LEN_KICKER_STATUSV2 12

#define CMD_KICKER_CHARGE_MANUAL 9
#define LEN_KICKER_CHARGE_MANUAL 6

#define CMD_KICKER_CHARGE_AUTO 10
#define LEN_KICKER_CHARGE_AUTO 2

#define CMD_KICKER_IR_LOG 11
#define LEN_KICKER_IR_LOG 8

// MOTOR
#define CMD_MOTOR_MOVE 0
#define LEN_MOTOR_MOVE 6

#define CMD_MOTOR_DRIBBLE 1
#define LEN_MOTOR_DRIBBLE 2

#define CMD_MOTOR_SET_PID_PARAMS 2
#define LEN_MOTOR_SET_PID_PARAMS 21

#define CMD_MOTOR_SET_MODE 3
#define LEN_MOTOR_SET_MODE 3

#define CMD_MOTOR_SET_MANUAL 4
#define LEN_MOTOR_SET_MANUAL 3

#define CMD_MOTOR_SET_PID_SP 5
#define LEN_MOTOR_SET_PID_SP 5

#define CMD_MOTOR_PID_LOG 6
#define LEN_MOTOR_PID_LOG 28

#define CMD_MOTOR_SET_PARAMS 7
#define LEN_MOTOR_SET_PARAMS 81

#define CMD_MOTOR_MOVE_V2 8
#define LEN_MOTOR_MOVE_V2 8

// MOVEMENT
#define CMD_MOVEMENT_LIS3_LOG_RAW 0
#define LEN_MOVEMENT_LIS3_LOG_RAW 12

#define CMD_MOVEMENT_LIS3_LOG 1
#define LEN_MOVEMENT_LIS3_LOG 24

#define CMD_MOVEMENT_SET_PARAMS 2
#define LEN_MOVEMENT_SET_PARAMS 48

// MULTICAST
#define CMD_MULTICAST_UPDATE_ALL 0
#define LEN_MULTICAST_UPDATE_ALL 65

#define CMD_MULTICAST_UPDATE_ALL_V2 1
#define LEN_MULTICAST_UPDATE_ALL_V2 76

typedef struct PACKED _KickerStatus
{
	uint16_t chg;
	uint16_t disStr;
	uint16_t disChip;
	uint16_t cap;
	uint16_t tempStr;
	uint16_t tempChip;
} KickerStatus;

typedef struct PACKED _KickerKickV1
{
	uint8_t device;
	uint8_t mode;
	uint16_t duration;
} KickerKickV1;

typedef struct PACKED _KickerCharge
{
	uint16_t time;
} KickerCharge;

typedef struct PACKED _KickerSetDutyCycle
{
	uint16_t on;
	uint16_t off;
} KickerSetDutyCycle;

typedef struct PACKED _CMDKickerSetAutoload
{
	uint8_t log;
	uint8_t mode;
} CMDKickerSetAutoload;

typedef struct PACKED _CMDKickerSetMaxCapLevel
{
	uint16_t max;
} CMDKickerSetMaxCapLevel;

typedef struct PACKED _KickerKickV2	// 4
{
	uint8_t device:4;
	uint8_t mode:4;
	uint8_t level;
	uint16_t duration;
} KickerKickV2;

typedef struct PACKED _KickerStatusV2	// 12
{
	uint16_t chg;
	uint16_t cap;
	uint16_t TDiode[2];
	uint16_t TIGBT[2];
} KickerStatusV2;

typedef struct PACKED _KickerChargeManual	// 6
{
	uint16_t tOnTicks;
	uint16_t tOffTicks;
	uint16_t duration; // in 100us steps
} KickerChargeManual;

typedef struct PACKED _KickerChargeAuto	// 2
{
	uint16_t maxCapLevel;
} KickerChargeAuto;

typedef struct PACKED _KickerIrLog	// 8
{
	uint16_t voltage[4];
} KickerIrLog;

typedef struct PACKED _MotorMove	// 6
{
	int16_t vx; // [mm/s]
	int16_t vy;	// [mm/s]
	int16_t vw;	//outer rotation speed [mm/s]
} MotorMove;

typedef struct PACKED _MotorMoveV2	// 8
{
	int16_t vx;
	int16_t vy;
	int16_t vw;
	int16_t vv;
} MotorMoveV2;

typedef struct PACKED _MotorDribble	// 2
{
	int16_t rpm;
} MotorDribble;

typedef struct PACKED _MotorSetPidParams // 21
{
	uint8_t id;
	int32_t Kp;
	int32_t Ki;
	int32_t Kd;
	int32_t slewMax;
	int32_t frictionKick;
} MotorSetPidParams;

typedef struct PACKED _MotorSetParams	// 81
{
	uint8_t mode;				// 1
	int32_t Kp[5];				// 20
	int32_t Ki[5];				// 20
	int32_t Kd[5];				// 20
	int32_t slewMax[5];			// 20
} MotorSetParams;

typedef struct PACKED _SystemSetLogs	// 9
{
	uint8_t motor[5];
	uint8_t kicker;
	uint8_t movement;
	uint8_t accel;
	uint8_t ir;
} SystemSetLogs;

typedef struct PACKED _MotorSetMode // 3
{
	uint8_t id;
	uint8_t mode;
	uint8_t log;
} MotorSetMode;

typedef struct PACKED _MotorSetManual	// 3
{
	uint8_t id;
	int16_t power;
} MotorSetManual;

typedef struct PACKED _MotorSetPidSp	// 5
{
	uint8_t id;
	int32_t setpoint;
} MotorSetPidSp;

typedef struct PACKED _MotorPidLog		// 28
{
	uint8_t id;
	int32_t latest;
	int32_t setpoint;
	int32_t pError;
	int32_t iError;
	int32_t dError;
	int32_t output;
	uint8_t overload;
	uint16_t eCurrent;
} MotorPidLog;

typedef struct PACKED _SystemStatusMovement	// 12
{
	int32_t vx;	// [mm/s]
	int32_t vy;
	int32_t vw;
} SystemStatusMovement;

typedef struct PACKED _SystemPowerLog	// 14
{
	uint16_t iVCC;
	uint16_t i5V;
	uint16_t i3V3;
	uint16_t u1;
	uint16_t u2;
	uint16_t u3;
	uint16_t u4;
} SystemPowerLog;

typedef struct PACKED _SystemSetIdentity	// 27
{
	uint32_t	cpuId[3];
	uint8_t		mac[6];
	uint8_t		ip[4];
	uint16_t	port;
	uint16_t	serverPort;
	uint8_t		botId;
} SystemSetIdentity;

typedef struct PACKED _SystemAnnouncement	// 12
{
	uint32_t	cpuId[3];
} SystemAnnouncement;

typedef struct PACKED _SystemPing	// 4
{
	uint32_t id;
} SystemPing;

typedef struct PACKED _SystemPong	// 4
{
	uint32_t id;
} SystemPong;

typedef struct PACKED _SystemBigPing	// 200
{
	uint32_t id;
	uint8_t dummy[196];
} SystemBigPing;

typedef struct PACKED _MulticastUpdateAll	// 65
{
	uint8_t botId[5];		// 5 * 1 = 5
	MotorMove move[5];		// 5 * 6 = 30
	KickerKickV2 kick[5];	// 5 * 4 = 20
	MotorDribble dribble[5];	// 5 * 2 = 10
} MulticastUpdateAll;

typedef struct PACKED _MulticastUpdateAllV2	// 76
{
	uint8_t botId[5];		// 5 * 1 = 5
	MotorMoveV2 move[5];	// 5 * 8 = 40
	KickerKickV2 kick[5];	// 5 * 4 = 20
	MotorDribble dribble[5];	// 5 * 2 = 10
	uint8_t useMove;		// 1 (bitfield)
} MulticastUpdateAllV2;

typedef struct PACKED _MovementLis3LogRaw	// 12
{
	int16_t iax;
	int16_t iay;
	int16_t iaz;
	int16_t oax;
	int16_t oay;
	int16_t oaz;
} MovementLis3LogRaw;

typedef struct PACKED _MovementLis3Log	// 24
{
	int32_t ax;
	int32_t ay;
	int32_t aw;
	int32_t vx;
	int32_t vy;
	int32_t vw;
} MovementLis3Log;

typedef struct PACKED _MovementSetParams	// 48
{
	int32_t Kp[3];		// 12
	int32_t Ki[3];		// 12
	int32_t Kd[3];		// 12
	int32_t slew[3];	// 12
} MovementSetParams;

#endif /* COMMANDS_H_ */
