/*
 * led.c
 *
 *  Created on: 30.10.2010
 *      Author: AndreR
 */

#include "led.h"

#include "stm32f10x.h"
#include "stm32f10x_gpio.h"

#include "FreeRTOS.h"
#include "task.h"

#define LED0 (1 << 4)
#define LED1 (1 << 2)

typedef struct _LEDGlobal
{
	uint16_t state;		// Bit 0-7 I2C LEDs, Bit 8-9 primary LEDs
} LEDGlobal;

LEDGlobal led;

void LEDInit()
{
	led.state = 0;

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Pin = LED0 | LED1;
	gpioInit.GPIO_Mode = GPIO_Mode_Out_PP;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &gpioInit);
}

void LEDSet(uint16_t state)
{
	led.state = state;

	if(led.state & 0x01)
		GPIOA->BRR = LED0;
	else
		GPIOA->BSRR = LED0;

	if(led.state & 0x02)
		GPIOA->BRR = LED1;
	else
		GPIOA->BSRR = LED1;
}

void LEDMustOn(uint16_t on)
{
	LEDSet(led.state | on);
}

void LEDMustOff(uint16_t off)
{
	LEDSet(led.state & ~(off));
}

void LEDToggle(uint16_t toggle)
{
	LEDSet(led.state ^ toggle);
}

void LEDMustOnBit(uint8_t bit)
{
	LEDMustOn(1 << bit);
}

void LEDMustOffBit(uint8_t bit)
{
	LEDMustOff(1 << bit);
}

void LEDToggleBit(uint8_t bit)
{
	LEDToggle(1 << bit);
}

uint16_t LEDGetState()
{
	return led.state;
}
