/*
 * console.h
 *
 *  Created on: 24.05.2011
 *      Author: AndreR
 */

#ifndef __CONSOLE_H__
#define __CONSOLE_H__

#include "stm32f10x.h"

#include "constants.h"

#include "util/fifo_packet_synced.h"
#include "util/fifo8_synced.h"

#include "FreeRTOS.h"
#include "semphr.h"

typedef void(*NotifyTxReady)();

typedef struct _Console
{
	FifoPacketSynced*	rxFifos[CONSOLE_MAX_INTERFACES];
	FifoSynced*			txFifos[CONSOLE_MAX_INTERFACES];
	NotifyTxReady		observer[CONSOLE_MAX_INTERFACES];

	uint8_t usedInterfaces;

	uint8_t cmdData[CONSOLE_CMD_SIZE];
	char	printData[CONSOLE_PRINT_SIZE];

	uint8_t cmdReady;

	xSemaphoreHandle hFifoInUse;
	xSemaphoreHandle hCmdDataInUse;
	xSemaphoreHandle hPrintDataInUse;
} Console;

void		ConsoleInit();
int16_t		ConsoleAddInterface(FifoPacketSynced* pRxFifo, FifoSynced* pTxFifo, NotifyTxReady readyCb);
uint16_t	ConsoleNumCmdsReady();
uint8_t		ConsoleIsCmdReady();
int16_t		ConsoleCmdProcessed();
int16_t		ConsoleCmpCmd(const char* cmd);
int16_t		ConsoleCmpCmdI16(const char* cmp, uint8_t argc, int16_t* argv);
int16_t		ConsoleCmpCmdI32(const char* cmp, uint8_t argc, int32_t* argv);
int16_t		ConsolePrint(const char* fmt, ...);
int16_t		ConsolePut(uint8_t* data, uint16_t length);
int16_t		ConsoleGetCmd(uint8_t* pDst, uint16_t bufLen, uint16_t* pBytesRead);

#endif
