/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */

package edu.dhbw.mannheim.tigers.robotcontrolutility.model;

import java.util.HashMap;


/**
 * This class creates correctly formatted commands.
 * 
 * @author Lukas
 * 
 */

public class ActionTranslator
{
	// --------------------------------------------------------------------------
	// --- instance variables ---------------------------------------------------
	// --------------------------------------------------------------------------
	private ActionSender	sender;
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	public ActionTranslator()
	{
		this.sender = new ActionSender();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	
	/**
	 * turn hashmap containing all single commands into an instance of ActionCommand
	 */
	public void translate(HashMap<String, Double> values)
	{
		double forward = 0;
		double backward = 0;
		double left = 0;
		double right = 0;
		double rotateLeft = 0;
		double rotateRight = 0;
		double pass = 0;
		double force = 0;
		double chipKick = 0;
		double dribble = 0;
		double arm = 0;
		
		try
		{
			forward = values.get("forward");
			backward = values.get("backward");
			left = values.get("left");
			right = values.get("right");
			rotateLeft = values.get("rotateLeft");
			rotateRight = values.get("rotateRight");
			pass = values.get("pass");
			force = values.get("force");
			chipKick = values.get("chipKick");
			dribble = values.get("dribble");
			arm = values.get("arm");
		} catch (NullPointerException e)
		{
			e.printStackTrace();
			System.err.println("HashMap does not have the right keys!");
		}
		
		// forward - positive; backward - negative;
		double translateY = forward - backward;
		// right - positive; left - negative;
		double translateX = right - left;
		// rotateRight - positive; rotateLeft - negative;
		double rotate = rotateRight - rotateLeft;
		// bam-hierarchy: shoot-pass-cross
		ActionCommand cmd = new ActionCommand(translateY, translateX, rotate, force, chipKick, pass, arm, dribble);
		sender.setNewCmd(cmd);
	}
	

	public boolean start(String serverName, int port)
	{
		return sender.startSending(serverName, port);
	}
	

	public void stop()
	{
		sender.stopSending();
	}
}