/*
 * *********************************************************
 * Copyright (c) 2009 DHBW Mannheim - Tigers Mannheim
 * Project: tigers-robotControlUtility
 * Date: 19.11.2010
 * Authors: Clemens Teichmann <clteich@gmx.de>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.robotcontrolutility.presenter;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.*;

import net.java.games.input.*;
import edu.dhbw.mannheim.tigers.robotcontrolutility.view.utils.TextAreaAppender;
import edu.dhbw.mannheim.tigers.robotcontrolutility.view.MainFrame;


/**
 * @author Clemens
 *
 */

public class RCUPresenter
{
	// --------------------------------------------------------------------------
	// --- class variables ------------------------------------------------------
	// --------------------------------------------------------------------------
	private static RCUPresenter			instance					= null;
	
	// --- view ---
	private MainFrame							view						= null;
	
	// --- logger ---
	private Logger								log						= Logger.getLogger(RCUPresenter.class.getName());
	
	// --- Controllers ---
	private ControllerEnvironment			cEnv						= ControllerEnvironment.getDefaultEnvironment();
	private Controller[]						cs;
	private List<AControllerPresenter>	controllerPresenterS	= new ArrayList<AControllerPresenter>();	
	
	// -----------------------------------------------------------------
	// ----- Constructor -----------------------------------------------
	// -----------------------------------------------------------------
	private RCUPresenter()
	{
		view = MainFrame.getInstance();
		view.setVisible(true);
		
		// --- configure logger - system (appender: textarea) ---
		Layout layout = new PatternLayout("%d{HH:mm:ss,SSS} %-5p [%t|%C{1}] %m%n");
		Logger.getRootLogger().addAppender(new TextAreaAppender(view.loggingTextArea, layout));
		
		// --- get Controllers and initiate Presenters ---
		this.setUpController();
	}
	

	// -----------------------------------------------------------------
	// ----- Methods ---------------------------------------------------
	// -----------------------------------------------------------------	
	public static RCUPresenter getInstance()
	{
		if (instance == null)
			instance = new RCUPresenter();
		return instance;
	}
	

	public void setUpController()
	{
		boolean controllerFound	= false;
		cs = cEnv.getControllers();
		
		// --- check if any controllers (keyboard, mouse, gamepad and so on) are available ---
		if (cs.length == 0)
		{
			System.out.println("No controllers found");
			System.exit(0);
		}
		else
			log.info("Num. controllers: " + cs.length);
		
		Controller.Type type;
		int index = 0;
		
		// --- iterate trough all controllers ---
		while (index < cs.length)
		{
			type = cs[index].getType();
			
			// --- set up gamepad ---
			if ((type == Controller.Type.GAMEPAD) || (type == Controller.Type.STICK))
			{
				GamePadPresenter newPresenter = new GamePadPresenter(cs[index]);
				controllerPresenterS.add(newPresenter);
				controllerFound = true;
			}
			// --- set up keyboard ---
			if (type == Controller.Type.KEYBOARD)
			{
				KeyboardPresenter newPresenter = new KeyboardPresenter(cs[index]);
				controllerPresenterS.add(newPresenter);
				controllerFound = true;
			}
			index++;
		}
		if (!controllerFound)
		{
			System.out.println("No game pad found");
			System.exit(0);
		}
	}
	/**
	 * Start or stop sending and polling.
	 * 
	 * @param true if start, false if stop
	 * @return Returns an ArrayList<Boolean>. True if single connection from controller to bot successful set up.
	 */
	public List<Boolean> onStartStopButtonPressed(boolean activeState)
	{
		List<Boolean> returnList = new ArrayList<Boolean>();
		
		// --- Start polling when start-button pressed ---
		if(activeState)
		{
			int i = 0;
			for(AControllerPresenter cP : controllerPresenterS)
			{
				returnList.add(true);
				if(!cP.startPolling())
				{
					returnList.set(i, false);
				}
				i++;
			}
		}
		
		// --- Stop polling when stop-button pressed ---
		else
		{
			for(AControllerPresenter cP : controllerPresenterS)
			{
				cP.stopPolling();
			}
		}
		return returnList;
	}
}
