package edu.dhbw.mannheim.tigers.sim.controller;

import java.util.List;

import edu.dhbw.mannheim.tigers.sim.engine.IBasicSimEngine;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.EPluginType;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.PluginID;
import edu.dhbw.mannheim.tigers.sim.gui.IGUI;


/**
 * This is the interface which provides the {@link IGUI} with every functionality on the model it needs
 * 
 * @author Gero
 */
public interface IInputController extends IBasicSimEngine { // , ICanvasInputController {

	// --------------------------------------------------------------------------
	// --- initialization -------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return The actual {@link IGUI}-implementation
	 */
	public IGUI getGUI();
	

	/**
	 * Reloads the data which contains model-parameter from file
	 */
	public void reloadModelData();
	

	/**
	 * Persists the current settings-object to the file-system
	 */
	public void persistSettings();
	

	/**
	 * @return A String-list of the {@link PluginID} of the currently loaded {@link EPluginType#CAM_OUTPUT} plugins
	 */
	public List<String> getLoadedCamOutputPlugins();
	

	/**
	 * @return A String-list of the {@link PluginID} of the currently loaded {@link EPluginType#VISUALIZATION} plugins
	 */
	public List<String> getLoadedVisualizationPlugins();
	

	/**
	 * Initiates a complete reload of all plugins: First unloads every plugin currently loaded, and then starts a new
	 * scan on the plugins-folder
	 */
	public void reloadPlugins();
	

	// --------------------------------------------------------------------------
	// --- application ----------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Initializes a application shutdown from the {@link IGUI}. But before shutdown, this will lead to a call to
	 * {@link ASubController#verifyAppExit()}
	 */
	public void exitApp();
}
