package edu.dhbw.mannheim.tigers.sim.controller;

import java.util.ArrayList;
import java.util.List;

import edu.dhbw.mannheim.tigers.sim.controller.MainController.Type;
import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.ISimEngine;
import edu.dhbw.mannheim.tigers.sim.engine.model.IModelDataProvider;
import edu.dhbw.mannheim.tigers.sim.engine.observer.ISimEngineObservable;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.EPluginType;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.IPluginProvider;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.PluginID;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimPluginException;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.IGUI;
import edu.dhbw.mannheim.tigers.sim.settings.ISettingsProvider;


/**
 * This {@link ASubController} handles events generated by the {@link IGUI} and passes them to the model (and
 * vice-versa).
 * 
 * @author Gero
 */
class InputController extends ASubController implements IInputController {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private IGUI	gui;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see InputController
	 * @param main
	 */
	InputController(MainController main) {
		super(main, Type.INPUT);
	}
	

	// --------------------------------------------------------------------------
	// --- application ----------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void exitApp() {
		main.doExitApp();
	}
	

	@Override
	public boolean verifyAppExit() {
		return true;
	}
	

	@Override
	public void prepareAppExit() {
		gui.close();
	}
	

	// --------------------------------------------------------------------------
	// --- simulation control ---------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return Returns the currently (or last) running engine
	 */
	private ISimEngine getEngine() {
		return ((SimEngineController) main.getController(Type.ENGINE)).getEngine();
	}
	

	@Override
	public void pause() {
		getEngine().pause();
	}
	

	@Override
	public void pauseSimulation() {
		getEngine().pauseSimulation();
	}
	

	@Override
	public void resume() {
		getEngine().resume();
	}
	

	@Override
	public void start() {
		// Create a new SimEngine every time
		((SimEngineController) main.getController(Type.ENGINE)).createSimEngine(gui.getSelectedMode());
		getEngine().start();
	}
	

	@Override
	public void stop() {
		if (gui.verifyEndOfSimulation()) {
			getEngine().stop();
		} else {
			gui.refreshEngineState(getEngine().getState());
		}
	}
	

	// --------------------------------------------------------------------------
	// --- canvas input ---------------------------------------------------------
	// --------------------------------------------------------------------------
	// private SimEngineCanvasImplementor canvasImplementor;
	

	// @Override
	// public SimEngineCanvasImplementor getCanvasImplementor() {
	// if (canvasImplementor == null) {
	// canvasImplementor = new SimEngineCanvasImplementor(getEngine());
	// }
	// return canvasImplementor;
	// }
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public IGUI getGUI() {
		return gui;
	}
	

	void setGUI(IGUI newGui) {
		this.gui = newGui;
	}
	

	@Override
	protected void onUpdate(ISimEngineObservable observable, ESimEngineState event) {
		gui.refreshEngineState(event);
		
		switch (event) {
			case STOPPED:
				observable.removeObserver(this);
				break;
		}
	}
	

	@Override
	public void reloadModelData() {
		try {
			IModelDataProvider.INSTANCE.reload();
		} catch (SimResourceException e) {
			log.error(e.getMessage(), e);
		}
	}
	

	@Override
	public void persistSettings() {
		ISettingsProvider.INSTANCE.persist();
	}
	

	@Override
	public List<String> getLoadedCamOutputPlugins() {
		List<PluginID> ids;
		try {
			ids = IPluginProvider.INSTANCE.getLoadedPluginsOf(EPluginType.CAM_OUTPUT);
		} catch (SimPluginException err) {
			log.error("Unnable to get cam-output plugins for GUI!", err);
			ids = new ArrayList<PluginID>();
		}
		
		return PluginID.toStringList(ids);
	}
	

	@Override
	public List<String> getLoadedVisualizationPlugins() {
		List<PluginID> ids;
		try {
			ids = IPluginProvider.INSTANCE.getLoadedPluginsOf(EPluginType.VISUALIZATION);
		} catch (SimPluginException err) {
			log.error("Unnable to get visualization plugins for GUI!", err);
			ids = new ArrayList<PluginID>();
		}
		
		return PluginID.toStringList(ids);
	}
	

	@Override
	public void reloadPlugins() {
		IPluginProvider.INSTANCE.reloadPlugins();
	}
}
