package edu.dhbw.mannheim.tigers.sim.engine;


import java.util.concurrent.Callable;
import java.util.concurrent.Future;

import com.jme.input.InputHandler;
import com.jme.renderer.Camera;
import com.jme.scene.Node;
import com.jme.system.DisplaySystem;
import com.jmex.physics.PhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.adapter.ASimEngineAdapter;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.ASimDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ASimEngineMode;
import edu.dhbw.mannheim.tigers.sim.engine.observer.ISimEngineObservable;


/**
 * This interface is meant to represent the {@link SimEngine}s core-functionality to the 'inner
 * circle' of classes (aka engine ;-) ), as {@link ASimEngineMode}, {@link ASimEngineAdapter},
 * {@link ASimDelegate} and so on. It can be referred to as the opposite of the {@link ISimEngine},
 * which represents the {@link SimEngine} to the <i>outer parts</i> of the application.
 * 
 * @author Gero
 */
public interface ICore extends IBasicSimEngine, ISimEngineObservable {

	// --------------------------------------------------------------------------
	// --- components -----------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return	The SimEngines camera
	 */
	public Camera getCamera();
	public void setCam(Camera newCam);
	

	/**
	 * @return	The currently active {@link InputHandler}
	 */
	public InputHandler getInputHandler();
	public void setInputHandler(InputHandler newInput);
	
	
	/**
	 * @return	The currently stored {@link PhysicsSpace}
	 */
	public PhysicsSpace getPhysicsSpace();
	public void setPhysicsSpace(PhysicsSpace space);
	
	
	public Node getStatics();
	public Node getDynamics();
	public Node getBalls();
	public Node getRobots();
	public Node getHUD();
   public Node getVisualizationRoot();

	
	// --------------------------------------------------------------------------
	// --- display --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return	The currently used display
	 */
	public DisplaySystem getDisplay();

	
	public int getDisplayWidth();
	public int getDisplayHeight();
	public int getDisplayDepth();
	public int getDisplayFreq();
	public boolean isFullscreen();
	
	
	// --------------------------------------------------------------------------
	// --- game task queue ------------------------------------------------------
	// --------------------------------------------------------------------------
	public <R> Future<R> enqueueUpdateTask(Callable<R> task);
	public void shutdownUpdateQueue();
	public <R> Future<R> enqueueRenderTask(Callable<R> task);
	
	
	// --------------------------------------------------------------------------
	// --- state ----------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return	@see {@link ACycleControlledEngine#getCurrentCycle()}
	 */
	public long getCurrentCycle();
	
	/**
	 * @return @see {@link ACycleControlledEngine#getCyclesPerSecond()}
	 */
	public int getCyclesPerSecond();
	
	/**
	 * @return @see {@link ACycleControlledEngine#getFramesPerSecond()}
	 */
	public int getFramesPerSecond();

	/**
	 * @return	@see {@link ACycleControlledEngine#isRenderCycle()}
	 */
	public boolean isRenderCycle();

	
	/**
	 * @return	The {@link ESimEngineState} the SimEngine is in
	 */
	public ESimEngineState getState();
	
	
	/**
	 * @return	Is in debug mode?
	 */
	public boolean isDebug();

	/**
	 * (Un-)set debug mode
	 */
	public void setDebug(boolean debug);
	
	
	/**
	 * Resets the timer (used for excluding the first (usually very expensive cycle(s) from timer-statistics)
	 */
	public void resetTimer();
}
