package edu.dhbw.mannheim.tigers.sim.engine.adapter;

import edu.dhbw.mannheim.tigers.sim.engine.ICore;
import edu.dhbw.mannheim.tigers.sim.engine.com.visualization.IVisualizationDataInterpreter;
import edu.dhbw.mannheim.tigers.sim.engine.com.visualization.VisualizationDataReceiver;
import edu.dhbw.mannheim.tigers.sim.engine.modes.IVisDataInterpreterProvider;
import edu.dhbw.mannheim.tigers.sim.engine.modes.display.DisplayMode;


/**
 * Simply forwards incoming data from the AI to the {@link DisplayMode}
 * 
 * @author Gero
 */
public class DisplayModeAdapter extends ASimEngineAdapter {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Input
	private final IVisDataInterpreterProvider	visProvider;
	private VisualizationDataReceiver			visReceiver;
	private IVisualizationDataInterpreter		visInterpreter;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see DisplayModeAdapter
	 * @param core
	 * @param visProvider
	 */
	public DisplayModeAdapter(ICore core, IVisDataInterpreterProvider visProvider) {
		super("DisplayModeAdapter", core);
		
		this.visProvider = visProvider;
		
		core.addObservers(this);
	}
	

	@Override
	protected void onStart() {
		
		// ##### Setup visualization input
		visInterpreter = visProvider.getVisualizationDataInterpreter();
		visReceiver = new VisualizationDataReceiver(visInterpreter);
		visReceiver.start();
	}
	

	@Override
	protected void onPause() {/* Not used */
	}
	

	@Override
	protected void onResume() {/* Not used */
	}
	

	@Override
	protected void onStop() {
		cleanup();
	}
	

	@Override
	public void cleanup() {
		if (visReceiver != null) {
			visReceiver.stop();
			visReceiver = null;
		}
	}
}
