package edu.dhbw.mannheim.tigers.sim.engine.com.bot;


import edu.dhbw.mannheim.tigers.sim.engine.com.ERoboCupTeam;
import edu.dhbw.mannheim.tigers.sim.engine.com.IRoboCupControlProvider;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ct.CTBotController;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.tiger.TigerBotController;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ABotModel;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBasicBotModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ct.ICTBotControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.tiger.ITigerBotControl;


/**
 * This factory provides different types of {@link ABotController}s for different implementations of {@link ABotModel}.
 * 
 * @author Gero
 */
public class BotControllerFactory implements IBotControllerFactory {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static BotControllerFactory	instance	= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see BotControllerFactory
	 */
	private BotControllerFactory() {
	}
	

	public static synchronized BotControllerFactory getInstance() {
		if (instance == null) {
			instance = new BotControllerFactory();
		}
		return instance;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public ABotController<?, ?, ?> createBotController(IBasicBotModelControl botControl,
			IRoboCupControlProvider controlProvider, int port, ERoboCupTeam team) {
		
		ABotController<?, ?, ?> result = null;
		
		switch (botControl.getBotType()) {
			
			case CT_MODEL:
				result = new CTBotController("Bot: " + botControl.getID(), port, controlProvider,
						(ICTBotControl) botControl);
				break;
			
			case PSEUDO_BOT:
				/* It's so pseudo, doesn't need any controller ;-) */
				break;
			
			case TIGER_MODEL:
				result = new TigerBotController((ITigerBotControl) botControl, port, controlProvider, team);
				break;
		}
		
		return result;
	}
}
