package edu.dhbw.mannheim.tigers.sim.engine.com.bot.ct;


import java.io.IOException;
import java.io.OutputStream;

import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ABotStatusTransmitter;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ICommunicator;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ACommand;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ct.CTPIDHistory;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ct.CTStatus;


/**
 * A {@link ABotStatusTransmitter}-implementation for the TIGER-bot, which sends the {@link CTStatus} and
 * the {@link CTPIDHistory} to the CS
 * 
 * @author Gero, Malte
 */
public class CTBotStatusTransmitter extends ABotStatusTransmitter<ACommand>
{
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private final ICTBotStatusProvider	botStatusProvider;
	private final OutputStream				outStream;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see CTBotStatusTransmitter
	 * @param communicator
	 * @param outputStream
	 * @param botStatusProvider
	 */
	public CTBotStatusTransmitter(ICommunicator<ACommand> communicator, OutputStream outputStream,
			ICTBotStatusProvider botStatusProvider)
	{
		super(communicator);
		
		this.botStatusProvider = botStatusProvider;
		this.outStream = outputStream;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public boolean writeOutput() throws IOException {
		
		// Send the CTBotStatus...
		tempStatusList.clear();
		botStatusProvider.getBotStatus(tempStatusList);

		synchronized (this)
		{
			for (ACommand status : tempStatusList) {
				send(status);
			}
		}
		
		// ...and the CTPIDHistory!
		CTPIDHistory[] pids = botStatusProvider.getPIDHistory();
		for (int i = 0; i < pids.length; i++) {
			outStream.write(pids[i].getTransferData());
		}
		
		return true;
	}
	

	@Override
	protected void closeOutput() throws IOException
	{
		outStream.close();
	}
	

	@Override
	public synchronized void send(ACommand msg) throws IOException
	{
		outStream.write(msg.getTransferData());
	}
}
