package edu.dhbw.mannheim.tigers.sim.engine.model;

import com.jme.math.FastMath;
import com.jmex.physics.impl.ode.DynamicPhysicsNodeImpl;
import com.jmex.physics.impl.ode.OdePhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.model.UserData.Value;

/**
 * Provides a common constructor and some supporting functions for any moving physics node.
 * 
 * @author Gero
 */
public class DynamicPhysicsSimNode extends DynamicPhysicsNodeImpl {

	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long	serialVersionUID	= 585726822180187951L;
	
	private final TimedModelLocation location = TimedModelLocation.EMPTY_MODEL_LOCATION.clone();
	

	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see DynamicPhysicsSimNode
	  * @param space
	  */
	public DynamicPhysicsSimNode(OdePhysicsSpace space) {
		super(space);
		
		ModelHelper.addToDefaultCollisionGroup(this);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	protected TimedModelLocation getTimedModelLocation() {
		location.set(this.getWorldTranslation(), this.getWorldRotation(), System.nanoTime());
		return location;
	}
	
	
	protected static float rad(float degree) {
		return degree * FastMath.DEG_TO_RAD;		// Radians: the "pure" angle
	}
	
	
	protected void makePickable(Value value) {
	   ModelHelper.makePickable(this, value);
	}
}
