package edu.dhbw.mannheim.tigers.sim.engine.model;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.apache.log4j.Logger;

import com.jme.bounding.BoundingBox;
import com.jme.curve.PolylineCurve;
import com.jme.image.Texture;
import com.jme.image.Texture.WrapMode;
import com.jme.math.FastMath;
import com.jme.math.Quaternion;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.renderer.Renderer;
import com.jme.scene.Node;
import com.jme.scene.Spatial;
import com.jme.scene.state.BlendState;
import com.jme.scene.state.MaterialState;
import com.jme.scene.state.TextureState;
import com.jme.system.DisplaySystem;
import com.jme.util.TextureManager;
import com.jme.util.export.Savable;
import com.jme.util.export.binary.BinaryImporter;
import com.jme.util.geom.BufferUtils;
import com.jmex.model.converters.FormatConverter;
import com.jmex.model.converters.MaxToJme;
import com.jmex.model.converters.ObjToJme;
import com.jmex.model.converters.X3dToJme;
import com.jmex.physics.CollisionGroup;
import com.jmex.physics.DynamicPhysicsNode;
import com.jmex.physics.PhysicsNode;
import com.jmex.physics.PhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.Sim;
import edu.dhbw.mannheim.tigers.sim.engine.model.UserData.Value;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ABotModel;
import edu.dhbw.mannheim.tigers.sim.engine.resources.ITextureProvider;
import edu.dhbw.mannheim.tigers.sim.engine.resources.TextureProvider.ETexIndex;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;


/**
 * A helper-class providing several functions and methods coming handy while working with the model.
 * 
 * @author Gero
 */
public class ModelHelper {
	
	private static Logger				log	= Logger.getLogger(ModelHelper.class);
	

	// --------------------------------------------------------------------------
	// --- rotation, rotation, rotation... --------------------------------------
	// --------------------------------------------------------------------------
	public static final Quaternion	y360	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 360,
																Vector3f.UNIT_Y.clone());
	public static final Quaternion	y270	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 270,
																Vector3f.UNIT_Y.clone());
	public static final Quaternion	y180	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 180,
																Vector3f.UNIT_Y.clone());
	public static final Quaternion	y90	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 90,
																Vector3f.UNIT_Y.clone());
	public static final Quaternion	y0		= new Quaternion().fromAngleAxis(0, Vector3f.UNIT_Y.clone());
	
	
	public static Quaternion yDeg(float degree) {
		return y(FastMath.DEG_TO_RAD * degree);
	}
	

	public static Quaternion y(float rad) {
		return new Quaternion().fromAngleAxis(rad, Vector3f.UNIT_Y);
	}
	
	public static final Quaternion	z360	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 360,
																Vector3f.UNIT_Z.clone());
	public static final Quaternion	z270	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 270,
																Vector3f.UNIT_Z.clone());
	public static final Quaternion	z180	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 180,
																Vector3f.UNIT_Z.clone());
	public static final Quaternion	z90	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 90,
																Vector3f.UNIT_Z.clone());
	public static final Quaternion	z0		= new Quaternion().fromAngleAxis(0, Vector3f.UNIT_Z.clone());
	
	
	public static Quaternion zDeg(float degree) {
		return z(FastMath.DEG_TO_RAD * degree);
	}
	

	public static Quaternion z(float rad) {
		return new Quaternion().fromAngleAxis(rad, Vector3f.UNIT_Z);
	}
	
	public static final Quaternion	x360	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 360,
																Vector3f.UNIT_X.clone());
	public static final Quaternion	x270	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 270,
																Vector3f.UNIT_X.clone());
	public static final Quaternion	x180	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 180,
																Vector3f.UNIT_X.clone());
	public static final Quaternion	x90	= new Quaternion().fromAngleAxis(FastMath.DEG_TO_RAD * 90,
																Vector3f.UNIT_X.clone());
	public static final Quaternion	x0		= new Quaternion().fromAngleAxis(0, Vector3f.UNIT_X.clone());
	
	
	public static Quaternion xDeg(float degree) {
		return x(FastMath.DEG_TO_RAD * degree);
	}
	

	public static Quaternion x(float rad) {
		return new Quaternion().fromAngleAxis(rad, Vector3f.UNIT_X);
	}
	

	// --------------------------------------------------------------------------
	// --- pickable -------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Needs to be called with the 'main' {@link DynamicPhysicsNode} as parameter to make it pickable!
	 * 
	 * @param node
	 */
	public static void makePickable(PhysicsNode node, Value value) {
		
		// Picking
		node.setUserData(UserData.PICKABLE, new UserData(Value.TRUE));
		node.setUserData(UserData.TYPE, new UserData(value));
	}
	

	/**
	 * @param node
	 * @return Whether the given {@link DynamicPhysicsNode} had been set "pickable"!
	 */
	public static boolean isPickable(PhysicsNode node) {
		
		boolean result = false;
		if (node.getUserData(UserData.PICKABLE) instanceof UserData) {
			UserData ud = (UserData) node.getUserData(UserData.PICKABLE);
			result = ud.equals(Value.TRUE);
		}
		
		return result;
	}
	

	/**
	 * Recursively climbs up the scene-graph
	 * 
	 * @param from this node to maximal
	 * @param depthUp
	 * @return
	 */
	public static ABotModel lookForBot(Node from, int depthUp) {
		
		if (depthUp == 0) {
			return null;
		}
		
		final Node parent = from.getParent();
		Savable sav = parent.getUserData(UserData.TYPE);
		if (sav != null && sav.equals(Value.BOT)) {
			return (ABotModel) parent;
		}
		
		return lookForBot(parent, --depthUp);
	}
	

	// --------------------------------------------------------------------------
	// --- collision ------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * 
	 * <b>!!! Taken from PhysicsSpace#createDynamicNode <br/>
	 * </b>
	 * Used to add the given node in den 'Default'-{@link CollisionGroup} of its {@link PhysicsSpace} .
	 * 
	 * @param node
	 * @return Whether node has actually been added to the group
	 */
	public static void addToDefaultCollisionGroup(DynamicPhysicsNode node) {
		
		node.setCollisionGroup(node.getSpace().getDefaultCollisionGroup());
	}
	

	// --------------------------------------------------------------------------
	// --- load model -----------------------------------------------------------
	// --------------------------------------------------------------------------
	public static Spatial loadModel(String folderName, String fileName) throws IOException {

		// Path
		String f = Sim.MODEL_PATH + folderName;
		if (f.charAt(f.length() - 1) != '/') {
			f += "/";
		}
		
		URL folder = new File(f).toURI().toURL();
		URL model = new File(f + fileName).toURI().toURL();
		
		// Determine model type
		int dotIndex = fileName.lastIndexOf('.');
		String suffix = fileName.substring(dotIndex + 1, fileName.length());
		suffix = suffix.toLowerCase();
		
		FormatConverter converter = null;
		
		// X3D
		if (suffix.equals("x3d")) {
			try {
				converter = new X3dToJme();
			} catch (InstantiationException err) {
				// Who cares... ;-)
			}
		}
		
		// 3dsMax
		if (suffix.equals("3ds")) {
			converter = new MaxToJme();
		}
		
		// obj
		if (converter == null || suffix.length() < 1 || suffix.equals("obj")) {
			// Try .obj if there's no suffix or anything else has been wrong
			converter = new ObjToJme();
		}
		
		
		// Convert
		converter.setProperty("mtllib", folder);
		converter.setProperty("texdir", folder);
		
		ByteArrayOutputStream BO = new ByteArrayOutputStream();
		converter.convert(model.openStream(), BO);
		
		final Spatial result = (Spatial) BinaryImporter.getInstance().load(new ByteArrayInputStream(BO.toByteArray()));
		
		result.setLocalScale(UnitConversion.cm(1) / UnitConversion.mm(1)); // Scale models that are designed in mms to
																									// cms!
		
		// Model bounds
		result.setModelBound(new BoundingBox());
		result.updateModelBound();
		
		return result;
	}
	

	// --------------------------------------------------------------------------
	// --- color, transparency and textures -------------------------------------
	// --------------------------------------------------------------------------
	public static void color(Spatial spatial, DisplaySystem display, ColorRGBA color) {
		
		final MaterialState materialState = display.getRenderer().createMaterialState();
		materialState.setEnabled(true);
		materialState.setDiffuse(color);
		
		if (color.a < 1) {
			final BlendState blendState = display.getRenderer().createBlendState();
			blendState.setEnabled(true);
			blendState.setBlendEnabled(true);
			blendState.setSourceFunction(BlendState.SourceFunction.SourceAlpha);
			blendState.setDestinationFunction(BlendState.DestinationFunction.OneMinusSourceAlpha);
			spatial.setRenderState(blendState);
			spatial.setRenderQueueMode(Renderer.QUEUE_TRANSPARENT);
		} else {
			spatial.setRenderQueueMode(Renderer.QUEUE_OPAQUE);
		}
		
		spatial.setRenderState(materialState);
	}
	

	public static void makeTransparent(Spatial spatial, DisplaySystem display) {
		
		BlendState netAS = display.getRenderer().createBlendState();
		netAS.setBlendEnabled(true);
		netAS.setSourceFunction(BlendState.SourceFunction.SourceAlpha);
		netAS.setDestinationFunction(BlendState.DestinationFunction.One);
		netAS.setTestEnabled(true);
		netAS.setTestFunction(BlendState.TestFunction.GreaterThan);
		netAS.setEnabled(true);
		
		spatial.setRenderState(netAS);
		
		spatial.setRenderQueueMode(Renderer.QUEUE_TRANSPARENT);
	}
	

	/**
	 * Calls {@link #applyTexture(Spatial, DisplaySystem, ETexIndex, Vector3f)} with <code>new Vector3f(x, y, 0)</code>
	 */
	public static void applyTexture(Spatial spatial, DisplaySystem display, ETexIndex texID, int x, int y) {
		applyTexture(spatial, display, texID, new Vector3f(x, y, 0));
	}
	

	/**
	 * Calls {@link #applyTexture(Spatial, DisplaySystem, ETexIndex, Vector3f)} with <code>repeat = null</code>
	 */
	public static void applyTexture(Spatial spatial, DisplaySystem display, ETexIndex texID) {
		applyTexture(spatial, display, texID, null);
	}
	

	/**
	 * @param spatial The spatial to apply the created {@link TextureState} to
	 * @param display The display the spatial is shown on
	 * @param texID The ID of the loaded texture in the {@link TextureManager}
	 * @param repeat How to repeat the texture (null if do not repeat at all)
	 */
	public static void applyTexture(Spatial spatial, DisplaySystem display, ETexIndex texID, Vector3f repeat) {
		// Texture...
		final TextureState netTS = display.getRenderer().createTextureState();
		
		try {
			Texture tex = ITextureProvider.INSTANCE.getTexture(texID);
			
			if (repeat != null) {
				tex.setWrap(WrapMode.Repeat);
				tex.setScale(repeat);
			}
			
			netTS.setTexture(tex);
		} catch (SimResourceException exc) {
			log.warn(exc.getMessage());
		}
		netTS.setEnabled(true);
		
		spatial.setRenderState(netTS);
	}
	

	/**
	 * This function draws a {@link PolylineCurve} between the given points and attaches it to parent.
	 * Especially useful for debugging purposes
	 * 
	 * @param parent
	 * @param from
	 * @param to
	 * @param color
	 * @return The created {@link PolylineCurve}, so callers may gain control over their line
	 */
	public static PolylineCurve draw2DLine(Node parent, Vector3f from, Vector3f to, ColorRGBA... color) {
		
		// Position
		final Vector3f[] points = new Vector3f[2];
		
		points[0] = from;
		points[1] = to;
		
		PolylineCurve curve = new PolylineCurve("DebugLine", points);
		
		parent.attachChild(curve);
		
		curve.setColorBuffer(BufferUtils.createFloatBuffer(color));
		curve.updateRenderState();
		
		return curve;
	}
}
