package edu.dhbw.mannheim.tigers.sim.engine.model.bot;


import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.data;

import com.jme.bounding.BoundingBox;
import com.jme.math.Quaternion;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jme.scene.shape.Cylinder;
import com.jme.system.DisplaySystem;
import com.jmex.physics.DynamicPhysicsNode;
import com.jmex.physics.geometry.PhysicsCylinder;
import com.jmex.physics.impl.ode.OdePhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.com.ERoboCupTeam;
import edu.dhbw.mannheim.tigers.sim.engine.model.ModelHelper;
import edu.dhbw.mannheim.tigers.sim.engine.model.data.bot.ITigerBotModelData;
import edu.dhbw.mannheim.tigers.sim.engine.model.material.RoboCupMaterial;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelException;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelResourceException;


/**
 * This is a very simple, immovable {@link ABotModel}-implementation which is only supposed to support the
 * testing of simple AI-scenarios.
 * 
 * @author Gero
 */
public class PseudoBot extends ABotModel {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long			serialVersionUID	= 6725461002355506187L;
	
	// Settings
	private final ITigerBotModelData	data;
	private final ColorRGBA				color;
	
	// Parts
	private final DynamicPhysicsNode	body;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Constructor
	 * 
	 * @param space
	 * @param display
	 * @param modelType
	 * @param name
	 * @param team
	 * @throws SimModelResourceException
	 * @throws SimModelException
	 */
	public PseudoBot(OdePhysicsSpace space, DisplaySystem display, ITigerBotModelData data, ColorRGBA color,
			ERoboCupTeam team)
			throws SimModelResourceException, SimModelException {
		super(space, display, EBotModelType.PSEUDO_BOT, EBotModelType.PSEUDO_BOT.toString(), team);
		
		this.data = data;
		this.color = color;
		
		addPhysicNode(body = space.createDynamicNode());
		
		makePickable(body);
		
		createPhysicalNode();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void createPhysicalNode() throws SimModelResourceException, SimModelException {
		
		final float bodyRadius = data.getRadius();
		final float bodyHeight = data.getHeight();
		final float bodyToGroundDist = data.getBodyToGroundDist();
		
		// Visual
		body.setName("Pseudo-Bot [body]");
		final Cylinder vBody = new Cylinder("Pseudo-Bot [body] Vis", 5, 25, bodyRadius, bodyHeight, true);
		vBody.setModelBound(new BoundingBox());
		vBody.updateModelBound();
		
		ModelHelper.color(vBody, display, color); // Color!
		
		body.attachChild(vBody);
		
		// Physic
		
		final PhysicsCylinder phBody = body.createCylinder("Bot-Body");
		phBody.setMaterial(RoboCupMaterial.BOT_WHEEL);
		phBody.setLocalScale(new Vector3f(bodyRadius, bodyRadius, bodyHeight));
		body.setMass(data.getMass());
		
		// Location
		body.setLocalTranslation(0, data(bodyHeight / 2 + bodyToGroundDist), 0); // Lift it a bit above the floor
		
		// Rotation
		final Quaternion turnUp = new Quaternion();
		turnUp.fromAngleNormalAxis(rad(90), Vector3f.UNIT_X);
		body.setLocalRotation(turnUp); // and turn it up!
		
		this.attachChild(body);
	}
	

	@Override
	public void doUpdate(long cycleNumber, float tpc) {
		notifyObservers(getCurrentBotStatus(body.getWorldTranslation(), body.getWorldRotation()));
	}
	

	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public float getHeight() {
		return data.getHeight() + data.getBodyToGroundDist();
	}
	

	@Override
	public Node getNode() {
		return this;
	}
	

	@Override
	public float getRadius() {
		return data.getRadius();
	}
}
