package edu.dhbw.mannheim.tigers.sim.engine.model.field;


import com.jme.bounding.BoundingBox;
import com.jme.math.Vector3f;
import com.jme.renderer.Renderer;
import com.jme.scene.shape.Box;
import com.jme.system.DisplaySystem;
import com.jmex.physics.StaticPhysicsNode;
import com.jmex.physics.impl.ode.OdePhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.model.ASimNode;
import edu.dhbw.mannheim.tigers.sim.engine.model.ModelHelper;
import edu.dhbw.mannheim.tigers.sim.engine.model.material.RoboCupMaterial;
import edu.dhbw.mannheim.tigers.sim.engine.resources.TextureProvider.ETexIndex;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelException;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelResourceException;


/**
 * Representing the ground of a Robocup-field
 * 
 * @author Gero
 */
public class FieldGround extends ASimNode {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long	serialVersionUID	= 3262849803817974458L;
	
	private final float			fieldLength;
	private final float			fieldWidth;
	private final float			fieldDepth;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public FieldGround(OdePhysicsSpace space, DisplaySystem display, float fieldWidth, float fieldLength,
			float fieldDepth) throws SimModelResourceException, SimModelException {
		super(space, display, "FieldGround");
		
		this.fieldLength = fieldLength;
		this.fieldWidth = fieldWidth;
		this.fieldDepth = fieldDepth;
		
		createPhysicalNode();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void createPhysicalNode() throws SimModelResourceException {
		
		StaticPhysicsNode groundNode = space.createStaticNode();
		

		float halfFieldWidth = fieldWidth / 2;
		float halfFieldLength = fieldLength / 2;
		
		Box simpleGround = new Box("simpleGround", new Vector3f(-halfFieldWidth, -fieldDepth, -halfFieldLength),
				new Vector3f(halfFieldWidth, 0, halfFieldLength));
		simpleGround.setModelBound(new BoundingBox());
		simpleGround.updateModelBound();
		
		groundNode.attachChild(simpleGround);
		
		ModelHelper.applyTexture(groundNode, display, ETexIndex.SIMPLE_FIELD);
		
		groundNode.setRenderQueueMode(Renderer.QUEUE_OPAQUE);
		groundNode.setMaterial(RoboCupMaterial.GROUND);
		groundNode.generatePhysicsGeometry();
		
		this.attachChild(groundNode);
	}
}
