package edu.dhbw.mannheim.tigers.sim.engine.modes.match;


import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.data;
import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.m;
import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.mm;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;

import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jmex.physics.impl.ode.OdePhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.ASimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.SimEngine;
import edu.dhbw.mannheim.tigers.sim.engine.adapter.RoboCupMatchControlAdapter;
import edu.dhbw.mannheim.tigers.sim.engine.com.ERoboCupTeam;
import edu.dhbw.mannheim.tigers.sim.engine.com.visualization.IVisualizationDataInterpreter;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.ASimDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.BasicInputDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.BasicInputDelegate.InputType;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.BotLabelDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.ControlBotDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.FreeCameraDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.MousePickingDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.ScreenshotDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.SimPhysicsDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.SteerableBall;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.VideoCaptureDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.VisualizationDataDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.model.ball.IBallModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.ball.RoboCupBall;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ABotModel;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.BotID;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.BotIDProvider;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBasicBotModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ASimEngineMode;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ESimEngineMode;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelException;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelResourceException;
import edu.dhbw.mannheim.tigers.sim.settings.ISettingsProvider;
import edu.dhbw.mannheim.tigers.sim.settings.data.IMatchModeProperties;


/**
 * This {@link ASimEngineMode} represents a RoboCup-SSL match between two teams
 * 
 * @author Gero
 */
public class MatchMode extends ASimEngineMode implements IMatchModeControl {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Delegates
	private SimPhysicsDelegate				physicsDelegate;
	private FreeCameraDelegate				camDelegate;
//	private TigersBannerDelegate			skyBoxDelegate;
	private BasicInputDelegate				basicInputDelegate;
	private SteerableBall					steerableBall;
	private MousePickingDelegate			mousePickingDelegate;
	private BotLabelDelegate				botLabelDelegate;
	private ScreenshotDelegate				screenshotDelegate;
	private ControlBotDelegate				controlBotDelegate;
	private VisualizationDataDelegate	visualizationDataDelegate;
	@SuppressWarnings("unused")	// Needs no explicit update!
	private VideoCaptureDelegate			videoDelegate;
	

	// Scene
	private RoboCupBall						ballModel;
	private ArrayList<ABotModel>			botModels;
	
	/** Initial location for the ball model */
	private static final Vector3f			ballInitLocation	= new Vector3f(0, m(0.5f), 0);
	
	private static final Vector3f			botInitLocation	= new Vector3f(0, 0, 0);
	
	private static final LinkedList<Vector3f>  leftBotInitLocs = new LinkedList<Vector3f>(){
		/**  */
		private static final long	serialVersionUID	= 1L;

		{
			add(new Vector3f(800,0,-200));
			add(new Vector3f(0,0,-800));
			add(new Vector3f(-800,0,-200));
			add(new Vector3f(400,0,-2000));
			add(new Vector3f(-400,0,-2000));
		}
	};
	private static final LinkedList<Vector3f>  rightBotInitLocs = new LinkedList<Vector3f>(){
		/**  */
		private static final long	serialVersionUID	= 1L;

		{
			add(new Vector3f(800,0,200));
			add(new Vector3f(0,0,800));
			add(new Vector3f(-800,0,200));
			add(new Vector3f(400,0,2000));
			add(new Vector3f(-400,0,2000));
		}
	};

	
	/** Counter to avoid collision problems when adding bots to the simulation */
	private int									rightBotCount		= 0;
	private int									leftBotCount		= 0;
	

	// Properties
	private final IMatchModeProperties	props;


	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public MatchMode(SimEngine core) {
		super(core, ESimEngineMode.MATCH);
		
		botModels = new ArrayList<ABotModel>();
		
		props = ISettingsProvider.INSTANCE.getSimProperties().getMatchModeProperties();
	}
	
	
	// --------------------------------------------------------------------------
	// --- current ASimEngineState ----------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void changeStateTo(ESimEngineState newState) {
		
		switch (newState) {
			case PAUSED_SIMULATION:
				currentState = paused;
				break;
			
			case RUNNING:
				currentState = running;
				break;
			
			default:
				super.changeStateTo(newState);
				break;
		}
	}
	
	private ASimEngineState	paused	= new ASimEngineState() {
													
		public void update(float interpolation, float tpc, float cps) {
			camDelegate.update(interpolation, tpc, cps);
//			skyBoxDelegate.update(interpolation, tpc, cps);
			basicInputDelegate.update(interpolation, tpc, cps);
			mousePickingDelegate.update(interpolation, tpc, cps);
			botLabelDelegate.update(interpolation, tpc, cps);
			screenshotDelegate.update(interpolation, tpc, cps);
			controlBotDelegate.update(interpolation, tpc, cps);
		};
	};
	
	private ASimEngineState	running	= new ASimEngineState() {
		
		public void update(float interpolation, float tpc, float cps) {
			physicsDelegate.update(interpolation, tpc, cps);
			camDelegate.update(interpolation, tpc, cps);
//			skyBoxDelegate.update(interpolation, tpc, cps);
			basicInputDelegate.update(interpolation, tpc, cps);
			matchDelegate.update(interpolation, tpc, cps);
			mousePickingDelegate.update(interpolation, tpc, cps);
			botLabelDelegate.update(interpolation, tpc, cps);
			screenshotDelegate.update(interpolation, tpc, cps);
			controlBotDelegate.update(interpolation, tpc, cps);
			visualizationDataDelegate.update(interpolation, tpc, cps);
		};
	};
	
	
	// --------------------------------------------------------------------------
	// --- delegates ------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void setupDelegates() {
		addDelegate(physicsDelegate = new SimPhysicsDelegate(core));
		addDelegate(camDelegate = new FreeCameraDelegate(core));
//		addDelegate(skyBoxDelegate = new TigersBannerDelegate(core));
		addDelegate(matchDelegate);
		addDelegate(basicInputDelegate = new BasicInputDelegate(core, InputType.FIRST_PERSON));
		addDelegate(steerableBall = new SteerableBall(core));
		addDelegate(mousePickingDelegate = new MousePickingDelegate(core));
		addDelegate(botLabelDelegate = new BotLabelDelegate(core, this));
		addDelegate(screenshotDelegate = new ScreenshotDelegate(core));
		addDelegate(controlBotDelegate = new ControlBotDelegate(core));
		addDelegate(visualizationDataDelegate = new VisualizationDataDelegate(core, this));
		addDelegate(videoDelegate = new VideoCaptureDelegate(core));
	}

	
	// --------------------------------------------------------------------------
	// --- adapters -------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void setupAdapters() {
		addAdapter(new RoboCupMatchControlAdapter(core, this));
	}
	
	
	private ASimDelegate	matchDelegate	= new ASimDelegate(core) {

		public void initSimulation() {
			// Reset the BotIDProvider (in case it has been used before by another mode)
	      BotIDProvider.getInstance().reset();
			
			// The field
			try {
				Node field = factory.createField(core.getPhysicsSpace(), core.getDisplay(), props.getFieldModelType());
				core.getStatics().attachChild(field);
			} catch (SimModelException exc) {
				log.error("Error while creating the RoboCupField", exc);
				core.stop();
			} catch (SimModelResourceException exc) {
				log.error("Error while loading model data!", exc);
			}
		};
		
		public void cleanup() {
	      
	      // Reset the BotIDProvider
	      BotIDProvider.getInstance().reset();
	   };
	};
	
	
	// --------------------------------------------------------------------------
	// --- IMatchModeControl ----------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public Future<List<IBasicBotModelControl>> addBotModelToSimulation(final int count, final String type,
			final ColorRGBA color, final ERoboCupTeam team, final boolean left) {
		
		return core.enqueueUpdateTask(new Callable<List<IBasicBotModelControl>>() {
			
			@Override
			public List<IBasicBotModelControl> call() throws Exception {
				
				final List<IBasicBotModelControl> controls = new ArrayList<IBasicBotModelControl>();
				for (int i = 1; i <= count; i++) {
					controls.add(addBotModelToSimulation(type, color, team, left));
				}
				
				return controls;
			};
		});
	}
	

	private IBasicBotModelControl addBotModelToSimulation(String type, ColorRGBA color, ERoboCupTeam team, boolean left) {
		
		// Add the new bot to the scene...
		final ABotModel newBot = factory.createBotModel(core.getPhysicsSpace(), core.getDisplay(),
				type, team, color);
		
		// Location, location, location...
		final Vector3f initLocation = new Vector3f(botInitLocation);
		initLocation.y = data(newBot.getHeight()) / 2;					// Lift it out of the ground...
//		System.out.println(data(newBot.getRadius()) * 4);
		
		if (left) {		// -Z
			Vector3f tmp = leftBotInitLocs.get(leftBotCount);
			initLocation.x = mm(tmp.x);
			initLocation.z = mm(tmp.z);
			leftBotCount++;
			//initLocation.z = data(newBot.getRadius()) * 4 * --leftBotCount;	// Move away from initLocation
		} else {																				// botCount times to avoid collisions
			Vector3f tmp = rightBotInitLocs.get(rightBotCount);
			initLocation.x = mm(tmp.x);
			initLocation.z = mm(tmp.z);
			rightBotCount++;
			//initLocation.z = data(newBot.getRadius()) * 4 * ++rightBotCount;
		}
		
		newBot.setLocation(initLocation);
		newBot.updateGeometricState(1, false);
		
		// Add to scene
		core.getRobots().attachChild(newBot);
		
		// and to the model-list! =)
		botModels.add(newBot);
		
		// and to the update-list
		addUpdateableModel(newBot);
		
		// Activate physics
		newBot.setActive(true);
		
		// Make it renderable
		newBot.updateRenderState(); 
		
		return newBot;
	}
	

	@Override
	public IBallModelControl getBallControl() {
		return ballModel;
	}
	

	@Override
	public List<? extends IBasicBotModelControl> getBotControls() {
		return new ArrayList<IBasicBotModelControl>(botModels);
	}
	

	@Override
	public Future<Boolean> removeBotModel(final BotID... ids) {
		
		return core.enqueueUpdateTask(new Callable<Boolean>() {
			
			@Override
			public Boolean call() throws Exception {
				
				boolean result = true;
				
				// Remove from both:
				Iterator<ABotModel> it = botModels.iterator();
				for (BotID id : ids) {
					while (it.hasNext()) {
						ABotModel model = it.next();
						if (model.getID().equals(id)) {
							
							// ...from scene
							result &= model.removeFromParent();
							
							// ...from updateableModels
							removeUpdateableModel(model);
							
							// ...from list
							it.remove();
							break;
						}
					}
				}
				
				return result;
			}
		});
	}
	

	@Override
	public Future<IBallModelControl> addBallToSimulation() {
		
		return core.enqueueUpdateTask(new Callable<IBallModelControl>() {
			
			@Override
			public IBallModelControl call() throws Exception {
				
				RoboCupBall newBall = factory.createRoboCupBall((OdePhysicsSpace) core
						.getPhysicsSpace(), core.getDisplay());
				
				// Location, location, location...
				newBall.setLocalTranslation(new Vector3f(ballInitLocation));
				
				steerableBall.setBall(newBall);
				ballModel = newBall;
				
				core.getBalls().attachChild(ballModel);
				newBall.updateRenderState();
				
				addUpdateableModel(newBall);
				
				return ballModel;
			}
		});
	}
	

	@Override
	public Future<Boolean> removeBall() {
		
		return core.enqueueUpdateTask(new Callable<Boolean>() {
			
			@Override
			public Boolean call() throws Exception {
				
				boolean result = ballModel.removeFromParent(); // Remove from scene...
				
				ballModel = null; // "null" reference the control
				
				return result;
			}
		});
	}


   @Override
   public IVisualizationDataInterpreter getVisualizationDataInterpreter() {
      return visualizationDataDelegate;
   }
}
