package edu.dhbw.mannheim.tigers.sim.engine.plugins.cam;


import java.util.List;
import java.util.Random;

import org.apache.commons.configuration.SubnodeConfiguration;

import edu.dhbw.mannheim.tigers.sim.engine.model.TimedModelLocation;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.PluginID;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.cam.DefaultTwoCamOutput.TwoCam.Side;
import edu.dhbw.mannheim.tigers.sumatra.model.data.MessagesRobocupSslDetection.SSL_DetectionBall;
import edu.dhbw.mannheim.tigers.sumatra.model.data.MessagesRobocupSslDetection.SSL_DetectionRobot;


/**
 * This class extends the {@link DefaultCamOutput} with an Gaussian-error on the objects coordinates.
 * 
 * @author Gero
 */
public class GaussianTwoCamOutput extends DefaultTwoCamOutput {
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	public static final PluginID	ID						= new PluginID("GAUSSIAN_TWO_CAM_OUTPUT");
	
	public static final float		DEFAULT_STAN_DEV	= 2;
	
	protected final Random			rand;
	protected float					stdDev;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public GaussianTwoCamOutput(SubnodeConfiguration config) {
		super(config);

		this.stdDev = config.getFloat("stdDev", DEFAULT_STAN_DEV);
		
		this.rand = new Random();
	}
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void createCameras(List<ACam> cameras) {
		final float length = getRobocupFieldData().getFieldLength() + getRobocupFieldData().getFieldBorderWidthL() * 2;
		final float width = getRobocupFieldData().getFieldWidth() + getRobocupFieldData().getFieldBorderWidthW() * 2;
		
		cameras.add(new GaussianTwoCam(0, Side.LEFT, length, width, getOffset()));
		cameras.add(new GaussianTwoCam(1, Side.RIGHT, length, width, getOffset()));
	}
	
	
	@Override
	public void checkConfigConsistency(SubnodeConfiguration config) {
		super.checkConfigConsistency(config);
		
		if (config.getProperty("properties.stdDev") == null) {
			config.addProperty("properties.stdDev", DEFAULT_STAN_DEV);
		}
	}
	

	@Override
	public PluginID getID() {
		return ID;
	}
	
	
	protected class GaussianTwoCam extends TwoCam {
		
		public GaussianTwoCam(int id, Side side, float length, float width, float offset) {
			super(id, side, length, width, offset);
		}

		
		@Override
		protected SSL_DetectionBall.Builder convertBall(TimedModelLocation ball) {
			SSL_DetectionBall.Builder result = super.convertBall(ball);
			
			result.setX((float) (result.getX() + rand.nextGaussian() * stdDev));
			result.setY((float) (result.getY() + rand.nextGaussian() * stdDev));
			result.setZ((float) (result.getZ() + rand.nextGaussian() * stdDev));
			
			return result;
		}
		
		
		@Override
		protected SSL_DetectionRobot.Builder convertRobot(int id, TimedModelLocation bot) {
			SSL_DetectionRobot.Builder result = super.convertRobot(id, bot);
			
			result.setX((float) (result.getX() + rand.nextGaussian() * stdDev));
			result.setY((float) (result.getY() + rand.nextGaussian() * stdDev));
			
			return result;
		}
	}
}
