package edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization;


import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.mm;

import org.apache.log4j.Logger;

import com.jme.curve.PolylineCurve;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jme.scene.Spatial;
import com.jme.scene.state.RenderState;
import com.jme.util.geom.BufferUtils;

import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.Poi;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.Vec3;


/**
 * Base class for any visual representation of {@link Poi}s
 * 
 * @author Gero
 */
public abstract class AVisualObject {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final Logger	LOG	= Logger.getLogger(AVisualObject.class);
	

	protected final Poi				poi;
	protected final Node				localRoot;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see AVisualObject
	  * @param poi
	  * @param parent
	  */
	AVisualObject(Poi poi, Node parent) {
		
		this.poi = poi;
//		this.localRoot = new Node();
		localRoot = parent;
//		parent.attachChild(localRoot);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Draws a single visual belonging to the given {@link Poi}
	 * 
	 * @param poi
	 * @return The drawn visual
	 */
	protected abstract AVisualObject drawSingle(Poi poi);
	

	/**
	 * This method is called when a visual is not longer needed.
	 */
	protected abstract void undraw();
	

	/**
	 * Used to be sure that the {@link RenderState} of the {@link Spatial}s gets updated
	 */
	protected abstract void updateRenderState();
	

	/**
	 * @param poi The {@link Poi} to get the X(sim)-coord from
	 * @return The X-Coord in terms of the Simulator
	 */
	protected static float x(Vec3 p) {
   	float result = p.getY();
   	if (Float.isNaN(result) || Float.isInfinite(result)) {
   		LOG.warn("Incoming vector contains NaN/infinite values!");
   		return 0;
   	}
      return mm(result);
   }
	

	/**
	 * @param poi The {@link Poi} to get the Z(sim)-coord from
	 * @return The Z-Coord in terms of the Simulator
	 */
	protected static float z(Vec3 p) {
   	float result = p.getX();
   	if (Float.isNaN(result) || Float.isInfinite(result)) {
   		LOG.warn("Incoming vector contains NaN/infinite values!");
   		return 0;
   	}
		return mm(result);
	}
	

	protected static Vector3f vectorFrom(Vec3 p, float y) {
		return new Vector3f(x(p), y, z(p));
	}
	

	protected static PolylineCurve draw2DLine(Vec3 from, Vec3 to, float height, ColorRGBA color) {
		return draw2DLine(vectorFrom(from, height), vectorFrom(to, height), color);
	}
	

	protected static PolylineCurve draw2DLine(Vector3f from, Vector3f to, ColorRGBA color) {
		
		// Position
		final Vector3f[] points = new Vector3f[2];
		points[0] = from;
		// log.debug("Point 0: " + points[0]);
		points[1] = to;
		// log.debug("Point 1: " + points[1]);
		
		final PolylineCurve curve = new PolylineCurve("PathCurve", points);
		
		// Color
		final ColorRGBA[] colors = new ColorRGBA[1];
		colors[0] = color;
		curve.setColorBuffer(BufferUtils.createFloatBuffer(colors));
		
		return curve;
	}
}
