package edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization;


import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.cm;

import com.jme.bounding.BoundingBox;
import com.jme.curve.PolylineCurve;
import com.jme.math.FastMath;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jme.scene.shape.Cylinder;
import com.jme.system.DisplaySystem;

import edu.dhbw.mannheim.tigers.sim.engine.model.CoordinateConversionHelper;
import edu.dhbw.mannheim.tigers.sim.engine.model.ModelHelper;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.Poi;


/**
 * Visual representation of a predicted bot position
 * 
 * @author Gero
 */
public class PredictedBot extends AVisualObject {
   
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
   // Constancts
   private final ColorRGBA    colorP            = ColorRGBA.green.clone();
   private final ColorRGBA    colorN            = ColorRGBA.magenta.clone();
   private final ColorRGBA    colorDir          = ColorRGBA.orange.clone();
   private static final float BOT_HEIGHT        = cm(6);
   private static final float BOT_RADIUS        = cm(2);
   private static final float BOT_DIR_LENGTH    = BOT_RADIUS * 3;
   private static final float BOT_HEIGHT_OFFSET = BOT_HEIGHT / 2 + cm(2);
   
   
   private final DisplaySystem display;

   // Visual
   private final Cylinder     cylinder;
   private PolylineCurve 		curve;
   
   
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
   /**
     * @see PredictedBot
     * @param poi
     * @param parent
     * @param display
     */
   PredictedBot(Poi poi, Node parent, DisplaySystem display) {
      super(poi, parent);
      
      this.display = display;
      
      // Create visual body
      cylinder = new Cylinder("Poi", 5, 25, BOT_RADIUS, BOT_HEIGHT, true);
      cylinder.setModelBound(new BoundingBox());
      cylinder.updateModelBound();
      
      cylinder.setLocalRotation(ModelHelper.x90.clone());
      
      localRoot.attachChild(cylinder);
   }
   

   @Override
   protected AVisualObject drawSingle(Poi poi) {
      
      // Adjust body color and position
      ColorRGBA color = null;
      float val = 0;
      if (poi.getValue() >= 0) {
         color = colorP;
         val = poi.getValue();
      } else {
         color = colorN;
         val = -poi.getValue();
      }
      
      color.a = 0.6f;
      color.a *= val == 0 ? 1 : val;
      ModelHelper.color(cylinder, display, color);
      
      final Vector3f start = cylinder.getLocalTranslation();
      start.set(x(poi.getPos()), BOT_HEIGHT_OFFSET, z(poi.getPos()));
      
      
      // Adjust orientation
      if (curve != null) {
         curve.removeFromParent();
      }
      
      final Vector3f lineEnd = new Vector3f(start);
      final float simDir = CoordinateConversionHelper.toTwoPiRange(poi.getDir()) - FastMath.HALF_PI;
      
      lineEnd.x = -FastMath.cos(simDir);
      lineEnd.z = FastMath.sin(simDir);
      lineEnd.multLocal(BOT_DIR_LENGTH);
      lineEnd.addLocal(start);
//      System.out.println(pos + " --> " + lineEnd);
      
      curve = draw2DLine(start, lineEnd, colorDir);
      localRoot.attachChild(curve);
      curve.updateRenderState();
      
      return this;
   }
   

   @Override
   protected void undraw() {
      cylinder.removeFromParent();
      
      if (curve != null) {
         curve.removeFromParent();
      }
   }
   

   @Override
   protected void updateRenderState() {
      cylinder.updateRenderState();
      
      if (curve != null) {
         curve.updateRenderState();
      }
   }
}
