package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;


import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;
import edu.dhbw.mannheim.tigers.sim.gui.swing.settings.PluginSettingsFrame;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * This action opens the {@link PluginSettingsFrame}
 * 
 * @author Gero
 */
public class OpenPluginSettingsAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long						serialVersionUID	= 8048273706185796524L;
	
	private static final String					TITLE					= "Plugin Settings";
	private static final String					DESCRIPTION			= "Opens the plugin-settings window";
	

	private static OpenPluginSettingsAction	instance				= null;
	private final SwingGUI							gui;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public static synchronized OpenPluginSettingsAction getInstance(SwingGUI gui) {
		if (instance == null) {
			EImage imgType = EImage.PLUGIN_EDIT;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new OpenPluginSettingsAction(gui, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(OpenPluginSettingsAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new OpenPluginSettingsAction(gui);
			}
		}
		return instance;
	}
	

	private OpenPluginSettingsAction(SwingGUI gui) {
		super(TITLE, DESCRIPTION, gui.getInputController());
		this.gui = gui;
	}
	

	private OpenPluginSettingsAction(SwingGUI gui, Icon icon) {
		super(TITLE, DESCRIPTION, gui.getInputController(), icon);
		this.gui = gui;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void actionPerformed(ActionEvent e) {
		synchronized (actionSync) {
			this.setEnabled(false);
			
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					try {
						gui.setEnabled(false);
						
						JFrame frame = new PluginSettingsFrame(gui);
						frame.addWindowListener(new WindowAdapter() {
							@Override
							public void windowClosed(WindowEvent e) {
								OpenPluginSettingsAction.this.setEnabled(true);
							}
						});
					} finally {
						gui.setEnabled(true);
					}
				}
			});
		}
	}
	
	
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		
	}
}
