package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;


import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;
import edu.dhbw.mannheim.tigers.sim.gui.swing.settings.SettingsFrame;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Opens the settings window.
 * 
 * @author Malte, Gero
 */
public class OpenSettingsWindowAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long						serialVersionUID	= -6132207608874623613L;
	
	private static final String					TITLE					= "Settings";
	private static final String					DESCRIPTION			= "Open the settings window";
	
	private static OpenSettingsWindowAction	instance				= null;
	private final SwingGUI							gui;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * 
	 * @param gui
	 * @return
	 * @throws SimResourceException
	 */
	public static OpenSettingsWindowAction getInstance(SwingGUI gui) {
		if (instance == null) {
			EImage imgType = EImage.BULLET_WRENCH;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new OpenSettingsWindowAction(gui, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(OpenSettingsWindowAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new OpenSettingsWindowAction(gui);
			}
		}
		return instance;
	}
	

	private OpenSettingsWindowAction(SwingGUI gui) {
		super(TITLE, DESCRIPTION, gui.getInputController());
		this.gui = gui;
	}
	

	private OpenSettingsWindowAction(SwingGUI gui, Icon icon) {
		super(TITLE, DESCRIPTION, gui.getInputController(), icon);
		this.gui = gui;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void actionPerformed(ActionEvent e) {
		synchronized (actionSync) {
			this.setEnabled(false);
			
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					try {
						gui.setEnabled(false);
						
						JFrame frame = new SettingsFrame(gui);
						frame.addWindowListener(new WindowAdapter() {
							@Override
							public void windowClosed(WindowEvent e) {
								OpenSettingsWindowAction.this.setEnabled(true);
							}
						});
					} finally {
						gui.setEnabled(true);
					}
				}
			});
		}
	}
	
	
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		
	}
}
