package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;


import java.awt.event.ActionEvent;

import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Calls {@link IInputController#reloadModelData()} for reloading all the data containing model-parameter from file
 * 
 * @author Gero
 * 
 */
public class ReloadModelDataAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long					serialVersionUID	= -4910009842241017894L;
	
	private static final String				TITLE					= "Reload";
	private static final String				DESCRIPTION			= "This reloads the model data";
	

	private static ReloadModelDataAction	instance;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public synchronized static ReloadModelDataAction getInstance(IInputController input) {
		if (instance == null) {
			EImage imgType = EImage.MODEL_RELOAD;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new ReloadModelDataAction(input, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(ReloadPluginsAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new ReloadModelDataAction(input);
			}
		}
		return instance;
	}
	

	private ReloadModelDataAction(IInputController input, Icon icon) {
		super(TITLE, DESCRIPTION, input, icon);
	}
	

	private ReloadModelDataAction(IInputController input) {
		super(TITLE, DESCRIPTION, input);
	}
	

	// --------------------------------------------------------------------------
	// --- methods ----------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		
		try {
			input.reloadModelData();
		} finally {
			this.setEnabled(true);
		}
	}
}
