package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;


import java.awt.event.ActionEvent;

import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Calls {@link IInputController#reloadPlugins()}
 * 
 * @author Gero
 */
public class ReloadPluginsAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long				serialVersionUID	= 5670396222326701054L;
	
	private static final String			TITLE					= "Reload plugins";
	private static final String			DESCRIPTION			= "Reload the plugins from './plugins'";
	
	private static ReloadPluginsAction	instance				= null;
	private final SwingGUI					gui;
	
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	public static synchronized ReloadPluginsAction getInstance(SwingGUI gui) {
		if (instance == null) {
			EImage imgType = EImage.PLUGIN_GO;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new ReloadPluginsAction(gui, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(ReloadPluginsAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new ReloadPluginsAction(gui);
			}
		}
		return instance;
	}
	

	private ReloadPluginsAction(SwingGUI gui) {
		super(TITLE, DESCRIPTION, gui.getInputController());
		this.gui = gui;
	}
	

	private ReloadPluginsAction(SwingGUI gui, Icon icon) {
		super(TITLE, DESCRIPTION, gui.getInputController(), icon);
		this.gui = gui;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		
		try {
			gui.setEnabled(false);
			input.reloadPlugins();
		} finally {
			gui.setEnabled(true);
			this.setEnabled(true);
		}
	}
}
