package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Calls {@link IInputController#resume()} / {@link IInputController#start()}
 * 
 * @author Gero
 * 
 */
public class SimStartResumeAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long				serialVersionUID	= -7897023428215285909L;
	
	private static final String			TITLE					= "Start";
	private static final String			DESCRIPTION			= "This starts a new/resumes a paused simulation";
	private boolean							running				= false;
	
	private static SimStartResumeAction	instance;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public synchronized static SimStartResumeAction getInstance(SwingGUI gui) {
		if (instance == null) {
			EImage imgType = EImage.CONTROL_START;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new SimStartResumeAction(gui, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(SimPauseFullAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new SimStartResumeAction(gui);
			}
		}
		return instance;
	}
	

	private SimStartResumeAction(SwingGUI gui) {
		super(TITLE, DESCRIPTION, gui.getInputController());
	}
	

	private SimStartResumeAction(SwingGUI gui, Icon icon) {
		super(TITLE, DESCRIPTION, gui.getInputController(), icon);
	}
	

	@Override
	public void doPerformSyncAction(ActionEvent event) {
		
		if (running) {
			input.resume();
		} else {
			input.start();
		}
	}
	

	/**
	 * @param running Whether the engine is running or not
	 */
	public void setRunning(boolean running) {
		this.running = running;
	}
}
