package edu.dhbw.mannheim.tigers.sim.gui.swing.settings;


import java.io.IOException;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.engine.SimEngineSettings;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;


/**
 * A window containing the graphical interface of the setting files.
 * Several Properties, such as Engine-, Sim-, or Modelsettings can be set and saved here.
 * 
 * @author Malte, Gero
 */
public class SettingsFrame extends ASettingsFrame {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long	serialVersionUID	= 336680742571764125L;
	
	private final Logger			log					= Logger.getLogger(getClass());
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see SettingsFrame
	 * @param gui
	 */
	public SettingsFrame(SwingGUI gui) {
		super(gui);
	}
	

	@Override
	protected void addSettingsPanels(List<ASettingsPanel> panels, SwingGUI gui) {
		panels.add(new MatchSettingsPanel(gui.getInputController()));
		panels.add(new CamSettingsPanel(gui.getInputController()));
		panels.add(new EngineSettingsPanel(gui.getInputController()));
		panels.add(new VisualizationSettingsPanel(gui.getInputController()));
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void onCancel() {
		close();
	}
	

	@Override
	protected void onOk() {
		for (ISettingsPanel panel : settingsPanels) {
			if (!panel.checkInput()) {
				JOptionPane.showMessageDialog(this, "Invalid display configuration combination", "Invalid Settings",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		
		// Input checked ==> apply!
		for (ISettingsPanel panel : settingsPanels) {
			panel.apply();
		}
		

		// Persist engine settings
		try {
			SimEngineSettings.getInstance().save();
		} catch (IOException ioe) {
			log.warn("Couldn't persist engine settings!", ioe);
		}
		
		// Persist other settings
		gui.getInputController().persistSettings();
		
		this.dispose();
	}
}
