package edu.dhbw.mannheim.tigers.sim.resources;

import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;


/**
 * This is the basic interface all classes providing access to resources have to implement.
 * 
 * @author Gero
 */
public interface IResourceProvider {
	
	/**
	 * Initializes the storage by loading the resources into the memory.
	 * 
	 * @throws SimResourceException If the resources can not be found
	 */
	public abstract void initialize() throws SimResourceException;
	

	/**
	 * Clears the storage
	 */
	public abstract void cleanup();
	

	/**
	 * Acts like a {@link #cleanup()} and {@link #initialize()}.
	 * 
	 * @throws SimResourceException
	 */
	public abstract void reload() throws SimResourceException;
}